@@ -201,6 +201,81 @@ gedit_notebook_grab_focus (GtkWidget *widget)
 	gtk_widget_grab_focus (tab);
 }
 
+static gint
+find_tab_num_at_pos (GtkNotebook *notebook,
+                     gint         screen_x,
+                     gint         screen_y)
+{
+	GtkNotebook *nb = GTK_NOTEBOOK (notebook);
+	GtkPositionType tab_pos;
+	GtkWidget *page;
+	GtkAllocation tab_allocation;
+	gint page_num = 0;
+
+	tab_pos = gtk_notebook_get_tab_pos (GTK_NOTEBOOK (notebook));
+
+	while ((page = gtk_notebook_get_nth_page (nb, page_num)))
+	{
+		GtkWidget *tab;
+		gint max_x, max_y, x_root, y_root;
+
+		tab = gtk_notebook_get_tab_label (nb, page);
+		g_return_val_if_fail (tab != NULL, -1);
+
+		if (!gtk_widget_get_mapped (GTK_WIDGET (tab)))
+		{
+			page_num++;
+			continue;
+		}
+
+		gdk_window_get_origin (gtk_widget_get_window (tab), &x_root, &y_root);
+
+		gtk_widget_get_allocation (tab, &tab_allocation);
+		max_x = x_root + tab_allocation.x + tab_allocation.width;
+		max_y = y_root + tab_allocation.y + tab_allocation.height;
+
+		if ((tab_pos == GTK_POS_TOP || tab_pos == GTK_POS_BOTTOM) && screen_x <= max_x)
+		{
+			return page_num;
+		}
+
+		if ((tab_pos == GTK_POS_LEFT || tab_pos == GTK_POS_RIGHT) && screen_y <= max_y)
+		{
+			return page_num;
+		}
+
+		page_num++;
+	}
+
+	return -1;
+}
+
+static gboolean
+gedit_notebook_button_press (GtkWidget *widget,
+                             GdkEventButton *event)
+{
+	GtkNotebook *nb = GTK_NOTEBOOK (widget);
+	gint tab_clicked;
+
+	if (event->type != GDK_BUTTON_PRESS ||
+	    event->button != 3 ||
+	    (event->state & gtk_accelerator_get_default_mod_mask ()) != 0)
+	{
+		return FALSE;
+	}
+
+	tab_clicked = find_tab_num_at_pos (nb, event->x_root, event->y_root);
+	if (tab_clicked < 0)
+	{
+		return FALSE;
+	}
+
+	/* switch to the page the mouse is over */
+	gtk_notebook_set_current_page (nb, tab_clicked);
+
+	return TRUE;
+}
+
 /*
  * We need to override this because when we don't show the tabs, like in
  * fullscreen we need to have wrap around too
@@ -251,22 +326,28 @@ gedit_notebook_switch_page (GtkNotebook *notebook,
 {
 	GeditNotebook *nb = GEDIT_NOTEBOOK (notebook);
 
-	GTK_NOTEBOOK_CLASS (gedit_notebook_parent_class)->switch_page (notebook, page, page_num);
-
 	if (!nb->priv->ignore_focused_page_update)
 	{
+		gint prev_page;
+		GtkWidget *previous_page;
+
+		prev_page = gtk_notebook_get_current_page (notebook);
+		previous_page = gtk_notebook_get_nth_page (notebook, prev_page);
+
 		/* Remove the old page, we dont want to grow unnecessarily
 		 * the list */
 		if (nb->priv->focused_pages)
 		{
 			nb->priv->focused_pages =
-				g_list_remove (nb->priv->focused_pages, page);
+				g_list_remove (nb->priv->focused_pages, previous_page);
 		}
 
 		nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
-		                                         page);
+		                                         previous_page);
 	}
 
+	GTK_NOTEBOOK_CLASS (gedit_notebook_parent_class)->switch_page (notebook, page, page_num);
+
 	/* give focus to the tab */
 	gtk_widget_grab_focus (page);
 }
@@ -291,20 +372,11 @@ static void
 smart_tab_switching_on_closure (GeditNotebook *nb,
 				GeditTab      *tab)
 {
-	gboolean jump_to;
-
-	jump_to = GPOINTER_TO_INT (g_object_get_data
-	                           (G_OBJECT (tab), "jump_to"));
-
-	if (!jump_to || !nb->priv->focused_pages)
-	{
-		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
-	}
-	else
+	if (nb->priv->focused_pages)
 	{
 		GList *l;
 		GtkWidget *child;
-		int page_num;
+		gint page_num;
 
 		/* activate the last focused tab */
 		l = g_list_last (nb->priv->focused_pages);
@@ -427,6 +499,7 @@ gedit_notebook_class_init (GeditNotebookClass *klass)
 	object_class->set_property = gedit_notebook_set_property;
 
 	gtkwidget_class->grab_focus = gedit_notebook_grab_focus;
+	gtkwidget_class->button_press_event = gedit_notebook_button_press;
 
 	notebook_class->change_current_page = gedit_notebook_change_current_page;
 	notebook_class->switch_page = gedit_notebook_switch_page;
@@ -547,25 +620,10 @@ gedit_notebook_add_tab (GeditNotebook *nb,
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), position);
-		g_object_set_data (G_OBJECT (tab), 
-				   "jump_to",
-				   GINT_TO_POINTER (jump_to));
-		
 		gtk_widget_grab_focus (GTK_WIDGET (tab));
 	}
 }
 
-static void
-remove_tab (GeditTab      *tab,
-            GeditNotebook *nb)
-{
-	gint position;
-
-	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
-
-	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
-}
-
 /**
  * gedit_notebook_move_tab:
  * @src: a #GeditNotebook
@@ -591,28 +649,11 @@ gedit_notebook_move_tab (GeditNotebook *src,
 
 	/* make sure the tab isn't destroyed while we move it */
 	g_object_ref (tab);
-	remove_tab (tab, src);
+	gtk_container_remove (GTK_CONTAINER (src), GTK_WIDGET (tab));
 	gedit_notebook_add_tab (dest, tab, dest_position, TRUE);
 	g_object_unref (tab);
 }
 
-/**
- * gedit_notebook_remove_tab:
- * @nb: a #GeditNotebook
- * @tab: a #GeditTab
- *
- * Removes @tab from @nb.
- */
-void
-gedit_notebook_remove_tab (GeditNotebook *nb,
-			   GeditTab      *tab)
-{
-	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	remove_tab (tab, nb);
-}
-
 /**
  * gedit_notebook_remove_all_tabs:
  * @nb: a #GeditNotebook
@@ -625,7 +666,7 @@ gedit_notebook_remove_all_tabs (GeditNotebook *nb)
 	GList *tabs, *t;
 
 	g_return_if_fail (GEDIT_IS_NOTEBOOK (nb));
-	
+
 	g_list_free (nb->priv->focused_pages);
 	nb->priv->focused_pages = NULL;
 
